@@ -25,16 +25,19 @@
 defined('MOODLE_INTERNAL') || die();
 
 function xmldb_tinymce_spellchecker_upgrade($oldversion) {
-    global $CFG, $DB;
-    require_once(__DIR__.'/upgradelib.php');
+    global $CFG;
 
-    $dbman = $DB->get_manager();
+    // Moodle v2.8.0 release upgrade line.
+    // Put any upgrade step following this.
 
-    if ($oldversion < 2012051800) {
-        tinymce_spellchecker_migrate_settings();
-        upgrade_plugin_savepoint(true, 2012051800, 'tinymce', 'spellchecker');
-    }
+    // Moodle v2.9.0 release upgrade line.
+    // Put any upgrade step following this.
 
+    // Moodle v3.0.0 release upgrade line.
+    // Put any upgrade step following this.
+
+    // Moodle v3.1.0 release upgrade line.
+    // Put any upgrade step following this.
 
     return true;
 }
