@@ -190,5 +190,33 @@ function xmldb_quiz_upgrade($oldversion) {
         upgrade_mod_savepoint(true, 2016032600, 'quiz');
     }
 
+    // Moodle v3.1.0 release upgrade line.
+    // Put any upgrade step following this.
+
+    if ($oldversion < 2016052301) {
+        // Find quizzes with the combination of require passing grade and grade to pass 0.
+        $gradeitems = $DB->get_records_sql("
+            SELECT gi.id, gi.itemnumber, cm.id AS cmid
+              FROM {quiz} q
+        INNER JOIN {course_modules} cm ON q.id = cm.instance
+        INNER JOIN {grade_items} gi ON q.id = gi.iteminstance
+        INNER JOIN {modules} m ON m.id = cm.module
+             WHERE q.completionpass = 1
+               AND gi.gradepass = 0
+               AND cm.completiongradeitemnumber IS NULL
+               AND gi.itemmodule = m.name
+               AND gi.itemtype = ?
+               AND m.name = ?", array('mod', 'quiz'));
+
+        foreach ($gradeitems as $gradeitem) {
+            $DB->execute("UPDATE {course_modules}
+                             SET completiongradeitemnumber = :itemnumber
+                           WHERE id = :cmid",
+                array('itemnumber' => $gradeitem->itemnumber, 'cmid' => $gradeitem->cmid));
+        }
+        // Quiz savepoint reached.
+        upgrade_mod_savepoint(true, 2016052301, 'quiz');
+    }
+
     return true;
 }
