@@ -84,6 +84,16 @@ abstract class question_behaviour {
         return substr(get_class($this), 11);
     }
 
+    /**
+     * Whether the current attempt at this question could be completed just by the
+     * student interacting with the question, before $qa->finish() is called.
+     *
+     * @return boolean whether the attempt can finish naturally.
+     */
+    public function can_finish_during_attempt() {
+        return false;
+    }
+
     /**
      * Cause the question to be renderered. This gets the appropriate behaviour
      * renderer using {@link get_renderer()}, and adjusts the display
@@ -192,9 +202,9 @@ abstract class question_behaviour {
             return array();
         }
 
-        $vars = array('comment' => PARAM_RAW);
+        $vars = array('comment' => PARAM_RAW, 'commentformat' => PARAM_INT);
         if ($this->qa->get_max_mark()) {
-            $vars['mark'] = question_attempt::PARAM_MARK;
+            $vars['mark'] = PARAM_RAW_TRIMMED;
             $vars['maxmark'] = PARAM_FLOAT;
         }
         return $vars;
@@ -281,10 +291,37 @@ abstract class question_behaviour {
     }
 
     /**
-     * @return question_possible_response[] where keys are subpartid or an empty array if no classification is possible.
+     * Classify responses for this question into a number of sub parts and response classes as defined by
+     * {@link \question_type::get_possible_responses} for this question type.
+     *
+     * @param string $whichtries         which tries to analyse for response analysis. Will be one of
+     *                                   question_attempt::FIRST_TRY, LAST_TRY or ALL_TRIES.
+     *                                   Defaults to question_attempt::LAST_TRY.
+     * @return (question_classified_response|array)[] If $whichtries is question_attempt::FIRST_TRY or LAST_TRY index is subpartid
+     *                                   and values are question_classified_response instances.
+     *                                   If $whichtries is question_attempt::ALL_TRIES then first key is submitted response no
+     *                                   and the second key is subpartid.
      */
-    public function classify_response() {
-        return $this->question->classify_response($this->qa->get_last_qt_data());
+    public function classify_response($whichtries = question_attempt::LAST_TRY) {
+        if ($whichtries == question_attempt::LAST_TRY) {
+            return $this->question->classify_response($this->qa->get_last_qt_data());
+        } else {
+            $stepswithsubmit = $this->qa->get_steps_with_submitted_response_iterator();
+            if ($whichtries == question_attempt::FIRST_TRY) {
+                $firsttry = $stepswithsubmit[1];
+                if ($firsttry) {
+                    return $this->question->classify_response($firsttry->get_qt_data());
+                } else {
+                    return $this->question->classify_response(array());
+                }
+            } else {
+                $classifiedresponses = array();
+                foreach ($stepswithsubmit as $submittedresponseno => $step) {
+                    $classifiedresponses[$submittedresponseno] = $this->question->classify_response($step->get_qt_data());
+                }
+                return $classifiedresponses;
+            }
+        }
     }
 
     /**
@@ -351,7 +388,7 @@ abstract class question_behaviour {
 
         // So, now we know the comment is the same, so check the mark, if present.
         $previousfraction = $this->qa->get_fraction();
-        $newmark = $pendingstep->get_behaviour_var('mark');
+        $newmark = question_utils::clean_param_mark($pendingstep->get_behaviour_var('mark'));
 
         if (is_null($previousfraction)) {
             return is_null($newmark) || $newmark === '';
@@ -440,15 +477,24 @@ abstract class question_behaviour {
         }
 
         if ($pendingstep->has_behaviour_var('mark')) {
-            $fraction = $pendingstep->get_behaviour_var('mark') /
-                            $pendingstep->get_behaviour_var('maxmark');
-            if ($pendingstep->get_behaviour_var('mark') === '') {
+            $mark = question_utils::clean_param_mark($pendingstep->get_behaviour_var('mark'));
+            if ($mark === null) {
+                throw new coding_exception('Inalid number format ' . $pendingstep->get_behaviour_var('mark') .
+                        ' when processing a manual grading action.', 'Question ' . $this->question->id .
+                        ', slot ' . $this->qa->get_slot());
+
+            } else if ($mark === '') {
                 $fraction = null;
-            } else if ($fraction > $this->qa->get_max_fraction() || $fraction < $this->qa->get_min_fraction()) {
-                throw new coding_exception('Score out of range when processing ' .
-                        'a manual grading action.', 'Question ' . $this->question->id .
-                                ', slot ' . $this->qa->get_slot() . ', fraction ' . $fraction);
+
+            } else {
+                $fraction = $mark / $pendingstep->get_behaviour_var('maxmark');
+                if ($fraction > $this->qa->get_max_fraction() || $fraction < $this->qa->get_min_fraction()) {
+                    throw new coding_exception('Score out of range when processing ' .
+                            'a manual grading action.', 'Question ' . $this->question->id .
+                            ', slot ' . $this->qa->get_slot() . ', fraction ' . $fraction);
+                }
             }
+
             $pendingstep->set_fraction($fraction);
         }
 
@@ -488,7 +534,7 @@ abstract class question_behaviour {
             $a->comment = '';
         }
 
-        $mark = $step->get_behaviour_var('mark');
+        $mark = question_utils::clean_param_mark($step->get_behaviour_var('mark'));
         if (is_null($mark) || $mark === '') {
             return get_string('commented', 'question', $a->comment);
         } else {
@@ -504,6 +550,20 @@ abstract class question_behaviour {
     public function summarise_finish($step) {
         return get_string('attemptfinished', 'question');
     }
+
+    /**
+     * Does this step include a response submitted by a student?
+     *
+     * This method should return true for any attempt explicitly submitted by a student. The question engine itself will also
+     * automatically recognise any last saved response before the attempt is finished, you don't need to return true here for these
+     * steps with responses which are not explicitly submitted by the student.
+     *
+     * @param question_attempt_step $step
+     * @return bool is this a step within a question attempt that includes a submitted response by a student.
+     */
+    public function step_has_a_submitted_response($step) {
+        return false;
+    }
 }
 
 
@@ -625,6 +685,11 @@ abstract class question_behaviour_with_save extends question_behaviour {
     }
 }
 
+abstract class question_behaviour_with_multiple_tries extends question_behaviour_with_save {
+    public function step_has_a_submitted_response($step) {
+        return $step->has_behaviour_var('submit') && $step->get_state() != question_state::$invalid;
+    }
+}
 
 /**
  * This helper class contains the constants and methods required for
